local MAJOR = "StarLibBuffer-1.0"
local MINOR = 1
assert(LibStub, MAJOR.." requires LibStub") 
local LibBuffer = LibStub:NewLibrary(MAJOR, MINOR)
if not LibBuffer then return end
local LibError = LibStub("StarLibError-1.0")
assert(LibError, MAJOR .. " requires StarLibError-1.0")

local pool = setmetatable({}, {__mode = "k"})

local objects = {}
local objectsDict = {}
local update
local frame = CreateFrame("Frame")

local DEFAULT_LIMIT = 50

if not LibBuffer.__index then
	LibBuffer.__index = LibBuffer
end

-- @name StarLibBuffer:New
-- @usage StarLibBuffer:New(visitor, rows, cols, yres, xres, layers)
-- @param size The buffer's size
-- @param errorLevel The verbocity level
-- @return A new StarLibBuffer object

function LibBuffer:New(size, errorLevel)
	
	local obj = next(pool)

	if obj then
		pool[obj] = nil
	else
		obj = {}
		obj.buffer = {}
	end

	setmetatable(obj, self)

	obj:Resize(size)
	obj.error = LibError:New(MAJOR, errorLevel)
	
	tinsert(objects, obj)
	objectsDict[obj] = obj

	return obj	
	
end

-- @name StarLibBuffer:Del
-- @usage StarLibBuffer:Del([lcd]) or object:Del()
-- @param lcd An optional lcd object
-- @return Nothing
function LibBuffer:Del(lcd)
	
	if lcd and type(lcd) == "table" then
		pool[lcd] = true
		for i, v in ipairs(objects) do
			if v == lcd then
				objectsDict[lcd] = nil
				tremove(objects, i)
			end
		end
	else
		pool[self] = true
		for i, v in ipairs(objects) do
			if v == lcd then
				objectsDict[lcd] = nil
				tremove(objects, i)
			end
		end
	end
end

function LibBuffer:AsString()
	local str = ""
	for i = 0, self:Size() - 1 do
		if self.buffer[i] then
			str = str .. self.buffer[i]
		else
			self.error:Print("Bad index: " .. i)
		end
	end
	return str
end

function LibBuffer:Resize(size)
	if self:Size() < size then
		for i = self:Size(), size do
			self.buffer[i] = ' '
		end
	end
	if self:Size() > size then
		for i = self:Size(), size, -1 do
			tremove(self.buffer, i)
		end
	end
end

function LibBuffer:Replace(pos, val)
	self.buffer[pos] = val
end

function LibBuffer:Clear()
	for i = 0, self:Size() - 1 do
		self.buffer[i] = ' '
	end
end

function LibBuffer:FromString(str)
	for i = 0, strlen(str) - 1 do
		self.buffer[i] = str:sub(i + 1, i + 1)
	end
end

function LibBuffer:Size()
	local count = 0
	for k in pairs(self.buffer) do
		count = count + 1
	end
	return count
end

function LibBuffer:Sub(num)
	local j = self:Size() - num
	local t = LibBuffer:New(j)
	for i = num, self:Size() - 1 do
		t.buffer[i - num] = self.buffer[i]
	end
	return t
end

function LibBuffer:Copy(other)
	for k, v in pairs(self.buffer) do
		self.buffer[k] = other.buffer[k]
	end
end

function LibBuffer:Merge(from, pos, len)
	local i = 0
	while i < len do
		self.buffer[i + pos] = from.buffer[i]
		i = i + 1
	end
end
